@@ -165,7 +165,12 @@ sub _write_apache_test_conf
         File::Spec->catfile( $conf{apache_dir}, 'mason_handler_CGI.pl' );
     my $mod_perl_handler =
         File::Spec->catfile( $conf{apache_dir}, 'mason_handler_mod_perl.pl' );
-    my $default_args_method = (Apache::test::have_module('Apache::Request') ? 'mod_perl' : 'CGI');
+    my $apreq_module = $conf{version} =~ m/^2\./ ? 'Apache2::Request' : 'Apache::Request';
+    my $apstat_module = $conf{version} =~ m/^2\./ ? 'Apache2::Status' : 'Apache::Status';
+    # Apache::test::have_module often stderrs about not finding libapreq.so
+    # Putting SERVER_ROOT/lib in LD_LIBRARY_PATH would suppress that, but
+    # that would have to be done before perl starts running.
+    my $default_args_method = (Apache::test::have_module($apreq_module) ? 'mod_perl' : 'CGI');
 
     my %multiconf;
     $multiconf{1}{comp_root} = File::Spec->catfile( $conf{comp_root}, 'multiconf1' );
@@ -289,6 +294,11 @@ ServerRoot $conf{apache_dir}
     SetHandler html-mason
   </Location>
 </IfDefine>
+
+<Location /perl-status>
+    SetHandler perl-script
+    PerlHandler $apstat_module
+</Location>
 EOF
 
 # Apache::Filter is reported to not work with mod_perl 2
@@ -391,7 +401,7 @@ sub make_request {
       || \$self->delayed_object_params('request', 'apache_req')
       || \$self->delayed_object_params('request', 'cgi_request');
     \$r->content_type( 'text/fooml' );
-    \$r->send_http_header unless \$mod_perl::VERSION >= 1.99;
+    \$r->send_http_header unless \$mod_perl2::VERSION >= 2.00;
     HTML::Mason::Exception::Abort->throw(error => 'foo', aborted_value => 200);
 }
 
@@ -1071,7 +1081,7 @@ sub ACTION_manifest
     $self->add_to_cleanup('MANIFEST');
 }
 
-sub ACTION_html_docs
+sub ACTION_custom_html_docs
 {
     my $self = shift;
 
@@ -1417,6 +1427,30 @@ sub _alter_httpd_conf
     close CONF or die "Can't close $params->{apache_config_file}: $!";
 }
 
+sub do_create_makefile_pl
+{
+    my $self = shift;
+    $self->SUPER::do_create_makefile_pl(@_);
+
+    open my $fh, 'Makefile.PL'
+        or die "Cannot open Makefile.PL: $!";
+
+    my $makefile = join '', <$fh>;
+
+    close $fh;
+
+    open $fh, '>Makefile.PL'
+        or die "Cannot write to Makefile.PL: $!";
+
+    print $fh <<"EOF";
+# Added by Mason::Build so we can find Mason::Build again
+use lib 'inc';
+$makefile
+EOF
+
+    close $fh;
+}
+
 # Copied from HTML::Mason::Tools
 sub load_pkg {
     my ($pkg, $nf_error) = @_;
