@@ -1,11 +1,11 @@
 package HTML::Mason;
-# Copyright (c) 1998-2003 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
-use 5.005;
+use 5.006_000;
 
-$HTML::Mason::VERSION = '1.28';
+$HTML::Mason::VERSION = '1.30';
 
 use HTML::Mason::Interp;
 
@@ -169,7 +169,16 @@ recipes as well as a sample Mason-based website.
 
 =head1 AUTHORS
 
-Jonathan Swartz <swartz@pobox.com>, Dave Rolsky <autarch@urth.org>, Ken Williams <ken@mathforum.org>
+Jonathan Swartz <swartz@pobox.com>, Dave Rolsky <autarch@urth.org>, Ken Williams <ken@mathforum.org>, John Williams <williams@tni.com>
+
+=head1 COPYRIGHT
+
+Copyright (c) 1998-2005 Jonathan Swartz.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+The full text of the license can be found in the LICENSE file included
+with this module.
 
 =head1 SEE ALSO
 
