@@ -1,10 +1,11 @@
-# Copyright (c) 1998-2003 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 
 package HTML::Mason::Compiler::ToObject;
 
 use strict;
+use warnings;
 
 use Params::Validate qw(SCALAR validate);
 use HTML::Mason::Tools qw(make_fh taint_is_on);
@@ -53,18 +54,17 @@ BEGIN
 }
 
 use HTML::Mason::MethodMaker
-    ( read_write => [ map { [ $_ => __PACKAGE__->validation_spec->{$_} ] }
-		      qw( comp_class
-                          define_args_hash
-                          in_package
-			  postamble
-			  preamble
-                          subcomp_class
-			  use_strict
-                        )
+    ( read_only => [
+		    qw( comp_class
+			define_args_hash
+			in_package
+			postamble
+			preamble
+			subcomp_class
+			use_strict
+			)
 		    ],
-    );
-
+      );
 
 sub compile
 {
@@ -123,15 +123,6 @@ sub compile_to_file
     return \@newfiles;
 }
 
-sub object_id
-{
-    my $self = shift;
-
-    local $self->{comp_class} = '';
-
-    return $self->SUPER::object_id;
-}
-
 sub _output_chunk
 {
     my ($self, $fh, $string) = (shift, shift, shift);
@@ -157,19 +148,12 @@ sub compiled_component
     local $c->{compiled_def} = $self->_compile_subcomponents if %{ $c->{def} };
     local $c->{compiled_method} = $self->_compile_methods if %{ $c->{method} };
 
-    # Create the file header to assert creatorship
-    my $id = $self->object_id;
-    $id =~ s,([\\']),\\$1,g;
-    $self->_output_chunk($p{fh}, \$obj_text, "# MASON COMPILER ID: $id\n");
-
     # Some preamble stuff, including 'use strict', 'use vars', and <%once> block
     my $header = $self->_make_main_header;
     $self->_output_chunk($p{fh}, \$obj_text, $header);
 
-
     my $params = $self->_component_params;
 
-    $params->{compiler_id} = "'$id'";
     $params->{load_time} = time;
 
     $params->{subcomps} = '\%_def' if %{ $c->{def} };
@@ -182,7 +166,7 @@ sub compiled_component
 	{
 	    my $key = "subcomponent_$name";
 	    $subs{$key} = $pref->{code};
-	    $pref->{code} = "sub {\nHTML::Mason::Request->instance->call_dynamic( '$key', \@_ )\n}";
+	    $pref->{code} = "sub {\nHTML::Mason::Request->instance->call_dynamic('$key',\@_)\n}";
 	}
 	while (my ($name, $pref) = each %{ $c->{compiled_method} } )
 	{
@@ -208,9 +192,6 @@ sub compiled_component
 
 
 
-    $params->{object_size} = 0;
-    $params->{object_size} += length for ($header, %$params);
-
     $self->_output_chunk($p{fh}, \$obj_text,
 			 $self->_constructor( $self->comp_class,
 					      $params ),
@@ -220,29 +201,6 @@ sub compiled_component
     return \$obj_text;
 }
 
-sub assert_creatorship
-{
-    my ($self, $p) = @_;
-    my $id;
-    if ($p->{object_code}) {
-	# Read the object code as a string
-
-	($id) = ${$p->{object_code}} =~ /\A# MASON COMPILER ID: (\S+)$/m
-	    or wrong_compiler_error "Couldn't find a Compiler ID in compiled code.";
-    } else {
-	# Open the object file and read its first line
-
-	my $fh = make_fh();
-	open $fh, $p->{object_file} or die "Can't read $p->{object_file}: $!";
-	($id) = <$fh> =~ /\A# MASON COMPILER ID: (\S+)$/m
-	    or wrong_compiler_error "Couldn't find a Compiler ID in $p->{object_file}.";
-	close $fh;
-    }
-    
-    wrong_compiler_error 'This object file was created by an incompatible Compiler or Lexer.  Please remove the component files in your object directory.'
-	unless $id eq $self->object_id;
-}
-
 sub _compile_subcomponents
 {
     my $self = shift;
@@ -354,16 +312,24 @@ sub _body
 {
     my $self = shift;
 
-    return join '', ( $self->_set_request,
-		      $self->preamble,
-                      $self->_arg_declarations,
+    return join '', ( $self->preamble,
+                      $self->_set_request,
+		      $self->_set_buffer,
+		      $self->_arg_declarations,
                       $self->_filter,
-		      "\$m->debug_hook( \$m->current_comp->path ) if ( \%DB:: );\n\n",
+		      "\$m->debug_hook( \$m->current_comp->path ) if ( HTML::Mason::Compiler::IN_PERL_DB() );\n\n",
 		      $self->_blocks('init'),
+
+		      # don't show warnings when appending undefined value to $_outbuf
+		      "\n{ no warnings 'uninitialized';\n",
 		      $self->{current_compile}{body},
+		      "\n}\n",
+
 		      $self->_blocks('cleanup'),
 		      $self->postamble,
-		      "return undef;\n",
+
+		      # don't return values implicitly
+		      "return;\n",
 		    );
 }
 
@@ -376,6 +342,17 @@ sub _set_request
     return 'local $' . $self->in_package . '::m = $HTML::Mason::Commands::m;' . "\n";
 }
 
+sub _set_buffer
+{
+    my $self = shift;
+
+    if ($self->enable_autoflush) {
+	return '';
+    } else {
+	return 'my $_outbuf = $m->{top_stack}->[HTML::Mason::Request::STACK_BUFFER];' . "\n";
+    }
+}
+
 my %coercion_funcs = ( '@' => 'HTML::Mason::Tools::coerce_to_array',
 		       '%' => 'HTML::Mason::Tools::coerce_to_hash',
 		     );
@@ -612,9 +589,10 @@ historical reasons, this defaults to C<HTML::Mason::Commands>.
 
 =item preamble
 
-Text given for this parameter is placed at the beginning of each component,
-but after the execution of any C<< <%once> >> block. See also
-L<postamble|HTML::Mason::Params/postamble>. The request will be available as C<$m> in preamble code.
+Text given for this parameter is placed at the beginning of each
+component, but after the execution of any C<< <%once> >> block. See
+also L<postamble|HTML::Mason::Params/postamble>. The request will be available as C<$m> in preamble
+code.
 
 =item postamble
 
@@ -643,6 +621,13 @@ arguments, which can save memory and slightly improve execution speed.
 
 =back
 
+=head1 ACCESSOR METHODS
+
+All of the above properties have read-only accessor methods of the
+same name. You cannot change any property of a compiler after it has
+been created (but you can create multiple compilers with different
+properties).
+
 =head1 METHODS
 
 This class is primarily meant to be used by the Interpreter object,
