@@ -1,10 +1,11 @@
-# Copyright (c) 1998-2003 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 
 package HTML::Mason::Component::Subcomponent;
 
 use strict;
+use warnings;
 
 use HTML::Mason::Component;
 
@@ -75,8 +76,8 @@ subcomponent (defined by c<< <%def> >>).
 
 =item owner
 
-Return the component object that this subcomponent or method was
-defined within.
+Returns the component object within which this subcomponent or method
+was defined.
 
 =back
 
