@@ -1,6 +1,7 @@
 package HTML::Mason::Exceptions;
 
 use strict;
+use warnings;
 
 use vars qw($VERSION);
 
@@ -107,14 +108,11 @@ sub isa_mason_exception
     my ($err, $name) = @_;
     return unless defined $err;
 
-    if ($name) {
-	my $class = "HTML::Mason::Exception::$name";
-	no strict 'refs';
-	die "no such exception class $class" unless defined(${"${class}::VERSION"});
-	return UNIVERSAL::isa($err, "HTML::Mason::Exception::$name");
-    } else {
-	return UNIVERSAL::isa($err, "HTML::Mason::Exception");
-    }
+    $name = $name ? "HTML::Mason::Exception::$name" : "HTML::Mason::Exception";
+    no strict 'refs';
+    die "no such exception class $name" unless defined(${"${name}::VERSION"});
+
+    return UNIVERSAL::isa($err, $name);
 }
 
 sub rethrow_exception
