@@ -1,4 +1,5 @@
 use strict;
+use warnings;
 
 # We need to define an Apache package or we might get strange errors
 # like "Can't locate package Apache for
@@ -159,15 +160,9 @@ sub http_header {
 }
 
 sub send_http_header {
-    my $self = shift;
-
-    print STDOUT $self->http_header;
-
-    $self->{http_header_sent} = 1;
+    print shift->http_header;
 }
 
-sub http_header_sent { shift->{http_header_sent} }
-
 # How do we know this under CGI?
 # sub get_basic_auth_pw {}
 # sub note_basic_auth_failure {}
@@ -265,14 +260,14 @@ sub no_cache {
 }
 
 sub print {
-    print @_;
+    shift->query->print(@_);
 }
 
 sub send_fd {
     my ($self, $fd) = @_;
     local $_;
-
-    print STDOUT while defined ($_ = <$fd>);
+    my $p = $self->query->can('print');
+    $p->($_) while defined ($_ = <$fd>);
 }
 
 # Should this perhaps throw an exception?
@@ -308,6 +303,7 @@ sub params {
 package HTML::Mason::FakeTable;
 # Analogous to Apache::Table.
 use strict;
+use warnings;
 
 sub new {
     my $class = shift;
@@ -361,6 +357,7 @@ sub do {
 package HTML::Mason::FakeTableHash;
 # Used by HTML::Mason::FakeTable.
 use strict;
+use warnings;
 
 sub TIEHASH {
     my $class = shift;
