@@ -1,10 +1,11 @@
-# Copyright (c) 1998-2003 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
 package HTML::Mason::Lexer;
 
 use strict;
+use warnings;
 
 use HTML::Mason::Exceptions( abbr => [qw(param_error syntax_error error)] );
 
@@ -97,31 +98,15 @@ sub lex
 	$current->{compiler}->start_component;
 	$self->start;
     };
-    # Call this out here because it may be needed to break circular
-    # refs inside the compiler
-    $current->{compiler}->end_component;
-
-    rethrow_exception $@;
-}
-
-sub object_id
-{
-    my $self = shift;
-
-    my @vals;
-    foreach my $k ( sort keys %{ $self->validation_spec } )
+    my $err = $@;
+    # Always call end_component, but throw the first error
+    eval
     {
-	next if $k eq 'container';
+        $current->{compiler}->end_component;
+    };
+    $err ||= $@;
 
-	push @vals, $k;
-	push @vals, ( UNIVERSAL::isa( $self->{$k}, 'HASH' )  ? map { $_ => $self->{$k}{$_} } sort keys %{ $self->{$k} } :
-		      UNIVERSAL::isa( $self->{$k}, 'ARRAY' ) ? sort @{ $self->{$k} } :
-		      $self->{$k} );
-    }
-
-    local $^W; # ignore undef warnings
-    # unpack('%32C*', $x) computes the 32-bit checksum of $x
-    return unpack('%32C*', join "\0", class => ref($self), @vals);
+    rethrow_exception $err;
 }
 
 sub start
@@ -375,23 +360,8 @@ sub match_named_block
     }
 }
 
-# starts with an alpha character or underscore, followed by any word
-# character
-my $flag;
-if ( $] >= 5.006 )
-{
-    # Unicode-friendly
-
-    # Use eval because 5.00503 complains even if it doesn't execute
-    # it.
-    eval '$flag = qr/[[:alpha:]_]\w*/;';
-}
-else
-{
-    # Like [a-zA-Z_] but respects locales
-    $flag = qr/[^\W\d]\w*/x;
-}
-
+# Like [a-zA-Z_] but respects locales
+my $flag = qr/[[:alpha:]_]\w*/;
 sub escape_flag_regex { $flag }
 
 sub match_substitute
@@ -480,9 +450,11 @@ sub match_comp_content_call_end
 {
     my $self = shift;
 
-    if ( $self->{current}{comp_source} =~ m,\G</&>,gc )
+    if ( $self->{current}{comp_source} =~ m,\G</&(.*?)>,gc )
     {
-        $self->{current}{compiler}->component_content_call_end;
+	my $call = $1 || '';
+        $self->{current}{compiler}->component_content_call_end( call_end => $call );
+	$self->{current}{lines} += $call =~ tr/\n//;
 
         return 1;
     }
@@ -675,4 +647,8 @@ in Mason, you must implement the methods listed above.  If you plan to
 use a custom Compiler class that you're writing, you can do whatever
 you want.
 
+We recommend that any parameters you add to Lexer be read-only,
+because the compiler object_id is only computed once on creation
+and would not reflect any changes to Lexer parameters.
+
 =cut
