@@ -1,6 +1,7 @@
 package HTML::Mason::Tests;
 
 use strict;
+use warnings;
 
 use Cwd;
 
@@ -13,8 +14,12 @@ use HTML::Mason::Tools qw(make_fh);
 
 use Getopt::Long;
 
+use Test::Builder ();
+
 use vars qw($VERBOSE $DEBUG @SHARED);
 
+my $Test = Test::Builder->new;
+
 $VERBOSE = $ENV{MASON_DEBUG} || $ENV{MASON_VERBOSE} || $ENV{TEST_VERBOSE};
 $DEBUG = $ENV{MASON_DEBUG};
 
@@ -78,6 +83,8 @@ EOF
               component => <<'EOF',
 My depth is <% $m->depth %>.
 
+I <% $m->is_subrequest ? 'am' : 'am not' %> a subrequest.
+
 The top-level component is <% $m->request_comp->title %>.
 
 My stack looks like:
@@ -124,9 +131,14 @@ if (defined($cmd_options{tests_class})) {
 my %tests_to_run;
 if ($cmd_options{tests_to_run}) {
     for ($cmd_options{tests_to_run}) { s/^\s+//; s/\s+$// }
-    my @tests_to_run = sort { $a <=> $b } split(/\s*,\s*/, $cmd_options{tests_to_run});
+    my @tests_to_run = split(/\s*,\s*/, $cmd_options{tests_to_run});
+    if (grep { /[^0-9]/ } @tests_to_run) {
+	@tests_to_run = sort { $a cmp $b } @tests_to_run;
+    } else {
+	@tests_to_run = sort { $a <=> $b } @tests_to_run;
+    }
     %tests_to_run = map { ($_, 1) } @tests_to_run;
-    printf ("Running only test%s %s\n", @tests_to_run == 1 ? "" : "s", join(", ", @tests_to_run));
+    $Test->diag(sprintf("Running only test%s %s\n", @tests_to_run == 1 ? "" : "s", join(", ", @tests_to_run)))
 }
 
 my %tests_to_skip;
@@ -134,7 +146,7 @@ if ($cmd_options{tests_to_skip}) {
     for ($cmd_options{tests_to_skip}) { s/^\s+//; s/\s+$// }
     my @tests_to_skip = split(/\s*,\s*/, $cmd_options{tests_to_skip});
     %tests_to_skip = map { ($_, 1) } @tests_to_skip;
-    printf ("Skipping test%s %s\n", @tests_to_skip == 1 ? "" : "s", join(", ", @tests_to_skip));
+    $Test->diag(printf ("Skipping test%s %s\n", @tests_to_skip == 1 ? "" : "s", join(", ", @tests_to_skip)));
 }
 
 sub new
@@ -186,9 +198,6 @@ sub add_test
     die "no name provided for test\n"
 	unless exists $p{name};
 
-    $self->{test_count}++;
-    
-    
     unless ( exists $p{path} )
     {
 	$p{path} = $p{call_path} || $p{name};
@@ -242,11 +251,9 @@ sub run
 
     if ($DEBUG)
     {
-	print "Will " . ( $self->{create} ? '' : 'not ' ) . "create 'expect' files\n";
+	$Test->diag( "Will " . ( $self->{create} ? '' : 'not ' ) . "create 'expect' files\n" );
     }
 
-    $self->{test_count} = 0;
-
     eval
     {
         # 1 indicates to be silent on missing directories
@@ -271,14 +278,14 @@ sub _make_dirs
 
     unless ( -d $self->comp_root )
     {
-	print "Making comp_root directory: $comp_root\n" if $DEBUG;
+	$Test->diag( "Making comp_root directory: $comp_root\n" ) if $DEBUG;
 	mkpath( $self->comp_root, 0, 0755 )
 	    or die "Unable to make base test directory '$comp_root': $!";
     }
 
     unless ( -d $self->data_dir )
     {
-	print "Making data_dir directory: $data_dir\n" if $DEBUG;
+	$Test->diag( "Making data_dir directory: $data_dir\n" ) if $DEBUG;
 	mkpath( $self->data_dir, 0, 0755 )
 	    or die "Unable to make base test directory '$data_dir': $!";
     }
@@ -336,7 +343,7 @@ sub _write_support_comps
 
     unless ( @{ $self->{support} } )
     {
-	print "No support comps to create\n" if $DEBUG;
+	$Test->diag( "No support comps to create\n" ) if $DEBUG;
 	return;
     }
 
@@ -362,7 +369,7 @@ sub _write_test_comp
     my $dir = File::Spec->catdir( $self->comp_root, $self->{name}, @path );
     unless ( -d $dir )
     {
-	diag("Making dir: $dir\n") if $DEBUG;
+	$Test->diag( "Making dir: $dir\n" ) if $DEBUG;
 	mkpath( $dir, 0, 0755 )
 	    or die "Unable to create directory '$dir': $!";
     }
@@ -377,14 +384,14 @@ sub write_comp
 
     unless (-d $dir)
     {
-	diag("Making dir: $dir\n") if $DEBUG;
+	$Test->diag( "Making dir: $dir\n" ) if $DEBUG;
 	mkpath( $dir, 0, 0755 )
 	    or die "Unable to create directory '$dir': $!";
     }
 
     my $real_file = File::Spec->catfile( $dir, $file );
 
-    diag("Making component $path at $real_file\n")
+    $Test->diag( "Making component $path at $real_file\n" )
 	if $DEBUG;
 
     my $fh = make_fh();
@@ -401,11 +408,11 @@ sub _run_tests
     my $self = shift;
 
     my $count = scalar @{ $self->{tests} };
-    print "\n1..$count\n";
+    $Test->plan( tests => $count );
 
     if ($VERBOSE)
     {
-	diag("Running $self->{name} tests ($count tests): $self->{description}\n");
+	$Test->diag( "Running $self->{name} tests ($count tests): $self->{description}\n" );
     }
 
     my $x = 1;
@@ -446,7 +453,7 @@ sub _run_tests
 		next;
 	    }
 	}
-	diag("Running $test->{name} (#$x): $test->{description}\n") if $VERBOSE;
+	$Test->diag( "Running $test->{name} (#$x): $test->{description}\n" ) if $VERBOSE;
 	$self->_make_component unless $test->{skip_component};
 	$self->_run_test;
 	$x++;
@@ -472,10 +479,10 @@ sub _make_main_interp
 
     if ($DEBUG && %interp_params)
     {
-	diag("Interp params:\n");
+	$Test->diag( "Interp params:\n" );
 	while ( my ($k, $v) = each %interp_params)
 	{
-	    diag("  $k => $v\n");
+	    $Test->diag( "  $k => $v\n" );
 	}
     }
 
@@ -500,7 +507,13 @@ sub _run_test
     my $interp = $self->_make_main_interp;
     $interp->out_method( sub { for (@_) { $self->{buffer} .= $_ if defined $_ } } );
 
-    eval { $self->_execute($interp) };
+    eval {
+	# Run pre_code if test has it - pass in interp
+	if ($test->{pre_code}) {
+	    $test->{pre_code}->($interp);
+	}
+	$self->_execute($interp);
+    };
 
     return $self->check_result($@);
 }
@@ -510,7 +523,7 @@ sub _execute
     my ($self, $interp) = @_;
     my $test = $self->{current_test};
 
-    diag("Calling $test->{name} test with path: $test->{call_path}\n") if $DEBUG;
+    $Test->diag( "Calling $test->{name} test with path: $test->{call_path}\n" ) if $DEBUG;
     $test->{pretest_code}->() if $test->{pretest_code};
     $interp->exec( $test->{call_path}, @{$test->{call_args}} );
 }
@@ -519,6 +532,10 @@ sub check_result {
     my ($self, $error) = @_;
     my $test = $self->{current_test};
 
+    local $HTML::Mason::Tests::TODO = $self->{current_test}{todo}
+        if exists $self->{current_test}{todo};
+    $Test->todo if exists $self->{current_test}{todo};
+
     if ($error)
     {
 	if ( $test->{expect_error} )
@@ -531,33 +548,35 @@ sub check_result {
 	    {
 		if ($VERBOSE)
 		{
-		    diag("Got error:\n$error\n...but expected something matching:",
-                         "$test->{expect_error}\n");
+		    $Test->diag( "Got error:\n$error\n...but expected something matching:\n$test->{expect_error}\n" );
 		}
 		return $self->_fail;
 	    }
 	}
 	else
 	{
-	    diag("Unexpected error running $test->{name}:\n$error") if $VERBOSE;
+	    $Test->diag( "Unexpected error running $test->{name}:\n$error" ) if $VERBOSE;
 	    return $self->_fail;
 	}
 
     }
     elsif ( $test->{expect_error} )
     {
-	diag("Expected an error matching '$test->{expect_error}' but no error occurred\n")
-          if $VERBOSE;
+	$Test->diag( "Expected an error matching '$test->{expect_error}' but no error occurred - got successful output:\n$self->{buffer}\n" ) if $VERBOSE;
 	return $self->_fail;
     }
 
     if ($self->{create})
     {
-	diag("Results for $test->{name}:\n$self->{buffer}\n") if $VERBOSE;
+	$Test->diag( "Results for $test->{name}:\n$self->{buffer}\n" );
 	return;
     }
 
-    my $success = $test->{skip_expect} ? 1 : $self->check_output( actual => $self->{buffer}, expect => $test->{expect} );
+    my $success =
+	( $test->{skip_expect} ?
+	  1 :
+	  $self->check_output( actual => $self->{buffer}, expect => $test->{expect} )
+	);
 
     $success ? $self->_success : $self->_fail;
 }
@@ -575,14 +594,13 @@ sub check_output
     } else {
 	# Whitespace at end can vary.  (Or rather, it is varying in the tests, and
 	# should be made not to vary, but I don't have time to fix it yet.)
-	
+
 	for ($p{actual}, $p{expect}) {  s/\s+$//  }
 	$same = ($p{actual} eq $p{expect});
     }
 
     if (!$same and $VERBOSE) {
-	diag("Got ...\n-----\n$p{actual}\n-----\n",
-             "... but expected ...\n-----\n$p{expect}\n-----\n");
+	$Test->diag( "Got ...\n-----\n$p{actual}\n-----\n   ... but expected ...\n-----\n$p{expect}\n-----\n" );
     }
     return $same;
 }
@@ -592,10 +610,7 @@ sub _fail
     my $self = shift;
     my $test = $self->{current_test};
 
-    $self->{test_count}++;
-
-    diag("Result for $self->{name}: $test->{name}\n");
-    print STDOUT "not ok $self->{test_count}\n";
+    $Test->ok( 0, $test->{name} );
 }
 
 sub _success
@@ -603,10 +618,7 @@ sub _success
     my $self = shift;
     my $test = $self->{current_test};
 
-    $self->{test_count}++;
-
-    diag("Result for $self->{name}: $test->{name}\n") if $VERBOSE;
-    print STDOUT "ok $self->{test_count}\n";
+    $Test->ok( 1, $test->{name} );
 }
 
 sub _skip
@@ -614,11 +626,7 @@ sub _skip
     my $self = shift;
     my $test = $self->{current_test};
 
-    $self->{test_count}++;
-
-    die "no test name for " . $self->{test_count} unless $test->{name};
-    diag("Result for $self->{name}: $test->{name}\n");
-    print STDOUT "ok $self->{test_count}  # skip Skipped by user\n";
+    $Test->skip;
 }
 
 #
@@ -642,7 +650,7 @@ sub rm_tree {
     } elsif (-f $path) {
 	unlink $path;
     } else {
-	warn "Can't find $path to remove"
+	$Test->diag( "Can't find $path to remove" )
             unless $silent;
     }
 }
@@ -654,17 +662,6 @@ sub _cleanup
     rm_tree( $self->base_path, $DEBUG, @_ );
 }
 
-sub diag {
-    my @msgs = @_;
-    for (@msgs) {
-        $_ = 'undef' unless defined;
-        s/^/# /gms;
-    }
-    local($\, $", $,) = (undef, ' ', '');
-    push @msgs, "\n" unless $msgs[-1] =~ /\n\Z/;
-    print STDERR @msgs;
-}
-
 1;
 
 __END__
@@ -807,6 +804,11 @@ method.
 
 Provide an HTML::Mason::Interp object to be used for the test.
 
+=item * todo
+
+If this is given, the test will be treated as a todo test, so it will
+be expected to fail.  This should be a string.
+
 =back
 
 One of the following three options is required:
