@@ -1,4 +1,4 @@
-# Copyright (c) 1998-2002 by Jonathan Swartz. All rights reserved.
+# Copyright (c) 1998-2005 by Jonathan Swartz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -11,6 +11,7 @@
 package HTML::Mason::Tools;
 
 use strict;
+use warnings;
 
 use Cwd;
 use File::Spec;
@@ -21,7 +22,18 @@ require Exporter;
 use vars qw(@ISA @EXPORT_OK);
 
 @ISA = qw(Exporter);
-@EXPORT_OK = qw(read_file read_file_ref url_escape paths_eq compress_path mason_canonpath make_fh taint_is_on load_pkg pkg_loaded absolute_comp_path);
+@EXPORT_OK = qw(can_weaken read_file read_file_ref url_escape paths_eq compress_path mason_canonpath make_fh taint_is_on load_pkg pkg_loaded absolute_comp_path checksum);
+
+# Is weaken available? Even under 5.6+, it might not be available on systems w/o a compiler.
+#
+BEGIN
+{
+    require Scalar::Util;
+
+    my $can_weaken = defined &Scalar::Util::weaken ? 1 : 0;
+
+    sub can_weaken () { $can_weaken }
+}
 
 # read_file($file, $binmode)
 # Return contents of file. If $binmode is 1, read in binary mode.
@@ -113,14 +125,14 @@ sub mason_canonpath {
     # Just like File::Spec::canonpath, but we're having trouble
     # getting a patch through to them.
     my $path = shift;
-    $path =~ s|/+|/|g;                               # xx////yy  -> xx/yy
-    $path =~ s|(/\.)+/|/|g;                          # xx/././yy -> xx/yy
+    $path =~ s|/+|/|g;                                 # xx////yy  -> xx/yy
+    $path =~ s|(?:/\.)+/|/|g;                          # xx/././yy -> xx/yy
     {
-	$path =~ s|^(\./)+||s unless $path eq "./";  # ./xx      -> xx
-	$path =~ s|^/(\.\./)+|/|s;                   # /../../xx -> xx
-	$path =~ s|/\Z(?!\n)|| unless $path eq "/";  # xx/       -> xx
-	$path =~ s|/[^/]+/\.\.$|| && redo;           # /xx/..    -> /
-	$path =~ s|[^/]+/\.\./|| && redo;            # /xx/../yy -> /yy
+	$path =~ s|^(?:\./)+||s unless $path eq "./";  # ./xx      -> xx
+	$path =~ s|^/(?:\.\./)+|/|s;                   # /../../xx -> xx
+	$path =~ s|/\Z(?!\n)|| unless $path eq "/";    # xx/       -> xx
+	$path =~ s|/[^/]+/\.\.$|| && redo;             # /xx/..    -> /
+	$path =~ s|[^/]+/\.\./|| && redo;              # /xx/../yy -> /yy
     }
     return $path;
 }
@@ -181,10 +193,30 @@ sub load_pkg {
     return 1;
 }
 
+# This code seems to be very fragile!  Please don't check in changes
+# unless you've tested it with Perl 5.00503, 5.6.1, and 5.8.0, or at
+# least tell Dave to run the tests.
+my $TaintIsOn;
 sub taint_is_on
 {
-    local $^W;
-    return not eval { "$0$^X" && kill 0; 1 };
+    return $TaintIsOn if defined $TaintIsOn;
+    return $TaintIsOn = _taint_is_on();
+}
+
+sub _taint_is_on
+{
+    if ( $] >= 5.008 )
+    {
+        # We have to eval a string because this variable name causes
+        # earlier Perls to not compile at all.
+        return eval '${^TAINT}' ? 1 : 0;
+    }
+    else
+    {
+        local $^W;
+        eval { "+$0$^X" && eval 1 };
+        return $@ ? 1 : 0;
+    }
 }
 
 sub make_fh
@@ -230,6 +262,18 @@ sub coerce_to_hash
     param_error "Cannot coerce $val to a hash";
 }
 
+# Adler32 algorithm
+sub checksum {
+    my ($str) = @_;
+    
+    my $s1 = 1;
+    my $s2 = 1;
+    for my $c (unpack("C*", $str)) {
+	$s1 = ($s1 + $c ) % 65521;
+	$s2 = ($s2 + $s1) % 65521;
+    }
+    return ($s2 << 16) + $s1;
+}
 
 1;
 
@@ -329,6 +373,10 @@ if this can't be done.
 This function is called from the generated component code as part of a
 component's argument handling.
 
+=item checksum
+
+Computes a simple checksum of a string. Used for Compiler::object_id.
+
 =back
 
 =cut
